#!/bin/bash

# Output Colors
ERROR='\033[0;31m'
WARNING='\033[0;33m'
SUCCESS='\033[0;32m'
FILE='\033[1;34m'
TEXT='\033[0m'


# Get Basedir
BASEDIR=$(pwd)
COREDIR="vendor/codepalm/wp-plugin-core"


# On Exit
on_exit() {
    echo ""
    echo "Clean up temporary build-directory ..."
    rm -rf ${BASEDIR}/build

    if [ -d "${BASEDIR}/build" ]; then 
        echo "Couldn't remove build-directory in ${FILE}${BASEDIR}/${TEXT}"
    else
        echo "Removed build-directory in ${FILE}${BASEDIR}/${TEXT}"
    fi
    echo ""
}

trap on_exit EXIT


# Get Plugin Variables from /[PLUGIN]/bin/plugin-conf.sh
if [ -f "${BASEDIR}/bin/plugin-conf.sh" ]; then
    echo "Read Conf-File from ${FILE}${BASEDIR}/bin/plugin-conf.sh${TEXT}"
    . "${BASEDIR}/bin/plugin-conf.sh"
else
    echo "${FILE}${BASEDIR}/bin/plugin-conf.sh${TEXT}${ERROR} not existing${TEXT}"
    exit 1
fi


# Parse SCSS
if [ -f "${BASEDIR}/${COREDIR}/bin/start-scss.sh" ]; then
    echo "Read SCSS-File from ${FILE}${BASEDIR}/${COREDIR}/bin/start-scss.sh${TEXT}"
    . "${BASEDIR}/${COREDIR}/bin/start-scss.sh"
else
    echo "${FILE}${BASEDIR}/${COREDIR}/bin/start-scss.sh${TEXT}${ERROR} not existing${TEXT}"
    exit 1
fi


# Start Composer Update
echo ""
echo "Start Composer Update ..."
composer update
echo "Composer Update finished"

# Check if any changes are made
echo ""
echo "Check if all changes are committed and pushed to Git ..."
if ! test -z "$(git status --porcelain)"; then
    echo "${ERROR}You haven't committed all your changes${TEXT}"
    echo "Use: \"git commit -m <COMMIT_MESSAGE>\""
    exit 1
fi

if test 0 -eq $( echo "$(git push -n --porcelain)" | grep -c "up to date" ); then
    echo "${ERROR}You haven't pushed all your changes${TEXT}"
    echo "Use: \"git push\""
    exit 1
fi

echo "${SUCCESS}All changes were committed and pushed to Git${TEXT}"


# Parse plugin.json
echo ""
if [ -f "${BASEDIR}/${COREDIR}/bin/create-plugin-json.sh" ]; then
    echo "Parse Plugin-JSON from File ${FILE}${BASEDIR}/${COREDIR}/bin/create-plugin-json.sh${TEXT}"
    . "${BASEDIR}/${COREDIR}/bin/create-plugin-json.sh"
else
    echo "${FILE}${BASEDIR}/${COREDIR}/bin/create-plugin-json.sh${TEXT}${ERROR} not existing${TEXT}"
    exit 1
fi


# Parse error.json
echo ""
if [ -f "${BASEDIR}/${COREDIR}/bin/create-error-json.sh" ]; then
    echo "Parse Error-JSON from File ${FILE}${BASEDIR}/${COREDIR}/bin/create-error-json.sh${TEXT}"
    . "${BASEDIR}/${COREDIR}/bin/create-error-json.sh"
else
    echo "${FILE}${BASEDIR}/${COREDIR}/bin/create-error-json.sh${TEXT}${ERROR} not existing${TEXT}"
    exit 1
fi


# Get Git Branch Name
BRANCH_NAME="$(git symbolic-ref HEAD 2>/dev/null)" ||
BRANCH_NAME="(unnamed branch)"     # detached HEAD
BRANCH_NAME=${BRANCH_NAME##refs/heads/}
BRANCH_NAME_CLEAN="$( echo "${BRANCH_NAME}" | tr '/' '-' )"
BRANCH_HASH="$( echo -n $BRANCH_NAME_CLEAN | md5sum | cut -d"-" -f1 - | cut -d" " -f1 - )"


# Get SSH-Password from User
echo ""
echo "Enter the SSH-Password for ${REMOTE_USER}@${REMOTE_HOST}: "
stty -echo
read SSH_PASSWORD
stty echo

echo "Try to connect to ${REMOTE_HOST} ...";
echo ""
sshpass -p ${SSH_PASSWORD} ssh -p 2222 ${REMOTE_USER}@${REMOTE_HOST} exit > /dev/null
SSH_CONNECTION=$?

if test ${SSH_CONNECTION} -eq 0; then
    echo "${SUCCESS}SSH-Connection with Remote-Server established${TEXT}";
else 
    echo "${ERROR}Could not connect to Remote-Server${TEXT}";
    exit 0
fi



# Initial Output
echo "${SUCCESS}"
echo "====================================================="
echo ""
echo "Starting Plugin Transfer of ${PLUGIN_NAME}::${BRANCH_NAME}"
echo ""
echo "====================================================="
echo "${TEXT}"


# Create build-directory
if [ ! -d "${BASEDIR}/build" ]; then
    echo "Create build directory in ${FILE}${BASEDIR}/${TEXT} ..."
    mkdir ${BASEDIR}/build
fi


# Create ZIP-Archive from current Development Branch
PLUGIN_FILE="${PLUGIN_NAME}-${BRANCH_NAME_CLEAN}.zip"

echo ""
echo "Create ${FILE}${PLUGIN_FILE}${TEXT} ..."
git archive --format=zip --prefix=${PLUGIN_NAME}/ ${BRANCH_NAME} --worktree-attributes > build/${PLUGIN_FILE}

if [ ! -f "build/${PLUGIN_FILE}" ]; then
    echo "${ERROR}Archive couldn't been created${TEXT}"
fi


# Change Remote-Directory to branch
REMOTE_ROOT_PATH="${REMOTE_PATH}"
REMOTE_PATH="${REMOTE_PATH}/${BRANCH_HASH}"

# Create Directory on Remote if it's not existing
if sshpass -p ${SSH_PASSWORD} ssh -p 2222 ${REMOTE_USER}@${REMOTE_HOST} "[ ! -d /directory ]"; then 
    echo "Create Directory ${FILE}/${REMOTE_PATH}/${TEXT} on ${REMOTE_HOST} ...";
    sshpass -p ${SSH_PASSWORD} ssh -p 2222 ${REMOTE_USER}@${REMOTE_HOST} "mkdir -p /${REMOTE_PATH}/"
fi 


# Move ZIP-File to external server
echo ""
if sshpass -p ${SSH_PASSWORD} ssh -p 2222 ${REMOTE_USER}@${REMOTE_HOST} "[ ! -f /${REMOTE_PATH}/${PLUGIN_FILE} ]"; then 
    echo "Transferring ${FILE}/${REMOTE_PATH}/${PLUGIN_FILE}${TEXT} ..."
else 
    echo "Overwrite ${FILE}/${REMOTE_PATH}/${PLUGIN_FILE}${TEXT} ..."
fi

sshpass -p ${SSH_PASSWORD} scp -P 2222 ${BASEDIR}/build/${PLUGIN_FILE} ${REMOTE_USER}@${REMOTE_HOST}:/${REMOTE_PATH}/ > /dev/null
TEST_ARCHIVE=$?

if test ${TEST_ARCHIVE} -eq 0; then
    echo "${SUCCESS}Plugin Archive transferred successfully!${TEXT}"
else 
    echo "${ERROR}Plugin Archive couldn't been transfered${TEXT}";
fi


# Move plugin.json-File to external server
echo ""
if sshpass -p ${SSH_PASSWORD} ssh -p 2222 ${REMOTE_USER}@${REMOTE_HOST} "[ ! -f /${REMOTE_PATH}/plugin.json ]"; then 
    echo "Transferring ${FILE}/${REMOTE_PATH}/plugin.json${TEXT} ..."
else 
    echo "Overwrite ${FILE}/${REMOTE_PATH}/plugin.json${TEXT} ..."
fi

sshpass -p ${SSH_PASSWORD} scp -P 2222 ${BASEDIR}/plugin.json ${REMOTE_USER}@${REMOTE_HOST}:/${REMOTE_PATH}/ > /dev/null
TEST_INFORMATION=$?

if test ${TEST_INFORMATION} -eq 0; then
    echo "${SUCCESS}Plugin Informations transferred successfully!${TEXT}"
else 
    echo "${ERROR}Plugin Informations couldn't been transfered${TEXT}";
fi


# Move error.json-File to external server
echo ""
if sshpass -p ${SSH_PASSWORD} ssh -p 2222 ${REMOTE_USER}@${REMOTE_HOST} "[ ! -f /${REMOTE_PATH}/error.json ]"; then 
    echo "Transferring ${FILE}/${REMOTE_PATH}/error.json${TEXT} ..."
else 
    echo "Overwrite ${FILE}/${REMOTE_PATH}/error.json${TEXT} ..."
fi

sshpass -p ${SSH_PASSWORD} scp -P 2222 ${BASEDIR}/error.json ${REMOTE_USER}@${REMOTE_HOST}:/${REMOTE_PATH}/ > /dev/null
TEST_INFORMATION=$?

if test ${TEST_INFORMATION} -eq 0; then
    echo "${SUCCESS}Error Informations transferred successfully!${TEXT}"
else 
    echo "${ERROR}Error Informations couldn't been transfered${TEXT}";
fi


# Transfer Images
echo ""
echo "Create ${FILE}assets.zip${TEXT} from ${FILE}/assets/${TEXT} ..."
zip ./build/assets.zip -r assets

sshpass -p ${SSH_PASSWORD} scp -P 2222 ./build/assets.zip ${REMOTE_USER}@${REMOTE_HOST}:/${REMOTE_PATH}/ > /dev/null

if sshpass -p ${SSH_PASSWORD} ssh -p 2222 ${REMOTE_USER}@${REMOTE_HOST} "[ -f /${REMOTE_PATH}/assets.zip ]"; then 
    echo "${SUCCESS}Transferred ${FILE}/${REMOTE_PATH}/assets.zip${SUCCESS} ...${TEXT}"
else 
    echo "${ERROR}Wasn't able to transfer ${FILE}/${REMOTE_PATH}/assets.zip${ERROR} ...${TEXT}"
fi

sshpass -p ${SSH_PASSWORD} ssh -p 2222 ${REMOTE_USER}@${REMOTE_HOST} "unzip -o /${REMOTE_PATH}/assets.zip -d /${REMOTE_PATH}/"

echo "Remove assets.zip from Remote-Server ..."
sshpass -p ${SSH_PASSWORD} ssh -p 2222 ${REMOTE_USER}@${REMOTE_HOST} "rm /${REMOTE_PATH}/assets.zip"
if sshpass -p ${SSH_PASSWORD} ssh -p 2222 ${REMOTE_USER}@${REMOTE_HOST} "[ ! -f /${REMOTE_PATH}/assets.zip ]"; then 
    echo "${SUCCESS}assets.zip was removed from ${REMOTE_PATH}${TEXT}"
else 
    echo "${ERROR}assets.zip wasn't removed from ${REMOTE_PATH}${TEXT}"
    echo "${ERROR}Please remove assets.zip via FileZilla or WinSCP ...${TEXT}"
fi


# Transfer PHP-Update Checker
echo ""
if sshpass -p ${SSH_PASSWORD} ssh -p 2222 ${REMOTE_USER}@${REMOTE_HOST} "[ ! -f /${REMOTE_ROOT_PATH}/plugin.php ]"; then 
    echo "Transfering ${FILE}/${REMOTE_ROOT_PATH}/plugin.php${TEXT} ..."
else 
    echo "Overwriting ${FILE}/${REMOTE_ROOT_PATH}/plugin.php${TEXT} ..."
fi

sshpass -p ${SSH_PASSWORD} scp -P 2222 ./plugin.php ${REMOTE_USER}@${REMOTE_HOST}:/${REMOTE_ROOT_PATH}/ > /dev/null

if sshpass -p ${SSH_PASSWORD} ssh -p 2222 ${REMOTE_USER}@${REMOTE_HOST} "[ -f /${REMOTE_ROOT_PATH}/plugin.php ]"; then 
    echo "${FILE}/${REMOTE_ROOT_PATH}/plugin.php${SUCCESS} was transfered successfully${TEXT}"
else 
    echo "${FILE}/${REMOTE_ROOT_PATH}/plugin.php${ERROR} couldn't be transfered${TEXT}"
    exit 1
fi


# Finished
echo "${SUCCESS}"
echo "====================================================="
echo ""
echo "We're finished. Have fun!"
echo "Cleanup-Process will start now"
echo ""
echo "=====================================================${TEXT}"

exit